#include <stdio.h>

#define N	10

int gauss_inverse(int n, double a[], double x[])
{
  double g[N][N+N];
  int i,j,k;
  double temp,gii,gik,gki;

  if (n>N) {
    fprintf(stderr,"gauss_inverse: dimension can't over %d\n",N);
    return(2);
  }

/*[g]=[a I] */
  for (i=0; i<n; i++) {
    for (j=0; j<n; j++) {
      g[i][j]=a[i*n+j];
      g[i][n+j]=0;
    }
    g[i][n+i]=1;
  }

/*Form upper triangle matrix */
  for (i=0; i<n; i++) {

/*  Find the row whose i-th element is not zero */
    for (k=i; k<n; k++) if (g[k][i]!=0) break;

/*  If i-th elements is all zero, it is singular maxtrix! */
    if (k==n) {
      fprintf(stderr,"gauss_inverse: can't solve singular singular!\n");
      return(1);
    }

/*  If the row is not the i-th, exchange the two rows */
    if (k!=i) {
      for (j=i; j<n+n; j++) {
        temp=g[i][j]; g[i][j]=g[k][j]; g[k][j]=temp;
      }
    }

/*  Normalize i-th row to Gii=1 */
    for (gii=g[i][i],j=i; j<n+n; j++) g[i][j]/=gii;

/*  Making the other i-th elements to zero */
    for (k=i+1; k<n; k++) {
      for (gki=g[k][i],j=i; j<n+n; j++) {
        g[k][j]-=g[i][j]*gki;
      }
    }
  }

/*Form diagonal matrix */
  for (i=n-1; i>=0; i--) {

/*  Making the other elements except the diagonal element to zero */
    for (k=i+1; k<n; k++) {
      gik=g[i][k];
      g[i][k]=0;
      for (j=n; j<n+n; j++) g[i][j]-=g[k][j]*gik;
    }
  }

/*Inverse Matrix is obtained from the right half matrix */
  for (i=0; i<n; i++) {
    for (j=0; j<n; j++) x[i*n+j]=g[i][n+j];
  }
  return(0);
}

